{
  Autor      : Michael Springwald
  Erstellt am: 17.09.2011
}

unit uobj;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Graphics, contnrs;

type
  { TPLBaseObj }
  TPLBaseObj = class
  private
    function GetItem(index: Integer): TBitmap;

  protected

  public
    Items:TObjectList;
    dir:string;
    constructor Create;
    destructor Destroy; override;

    procedure AddBitMap(const FileName:String; const Transparent:Boolean = True);
    procedure Paint(Canvas:TCanvas; const Left, Top:Integer); virtual; abstract;

    property Item[index:Integer]:TBitmap read GetItem;
  published

  end; // TPLBaseObj

    { TPLFloorPanels }
    TPLFloorPanels = class(TPLBaseObj)
    private

    protected

    public
      constructor Create;
      destructor Destroy; override;
    published

    end; // TPLFloorPanels

      { TPLNormalPanel }
      TPLNormalPanel = class(TPLFloorPanels)
      private

      protected

      public
        constructor Create;
        destructor Destroy; override;

        procedure Paint(Canvas:TCanvas;  const Left, Top:Integer); override;
      published

      end; // TPLNormalPanel

      { TPLSpikes }
      TPLSpikes = class(TPLFloorPanels)
      private

      protected

      public
        constructor Create;
        destructor Destroy; override;

        procedure Paint(Canvas:TCanvas;  const Left, Top:Integer); override;
      published

      end; // TPLSpikes

      { TPLSkeleton }
      TPLSkeleton = class(TPLFloorPanels)
      private

      protected

      public
        constructor Create;
        destructor Destroy; override;

        procedure Paint(Canvas:TCanvas;  const Left, Top:Integer); override;
      published

      end; // TPLSkeleton

    { TPLBigpillar }
    TPLBigpillar = class(TPLBaseObj)
    private

    protected

    public
      constructor Create;
      destructor Destroy; override;

      procedure Paint(Canvas:TCanvas;  const Left, Top:Integer); override;
    published

    end; // TPLBigpillar

    { TPLDoor }
    TPLDoor = class(TPLBaseObj)
    private
      fStatus: Integer;
    protected

    public
      constructor Create;
      destructor Destroy; override;
      procedure Paint(Canvas:TCanvas;  const Left, Top:Integer); override;
      property Status:Integer read fStatus write fStatus;
    published

    end; // TPLDoor

    { TPLKid }
    TPLKid = class(TPLBaseObj)
    private

    protected

    public
      constructor Create;
      destructor Destroy; override;

      procedure Paint(Canvas:TCanvas;  const Left, Top:Integer); override;
    published

    end; // TPLKid


var
  DataDir:String = '';

implementation

function TPLBaseObj.GetItem(index: Integer): TBitmap;
begin
  result:=items[index] as TBitmap;
end; // TPLBaseObj.GetItem

{ TPLBaseObj }
constructor TPLBaseObj.Create;
begin
  inherited Create;
  Items:=TObjectList.Create;
end; // TPLBaseObj.Create

destructor TPLBaseObj.Destroy;
begin
  Items.Free;
  inherited Destroy;
end; // TPLBaseObj.Destroy

procedure TPLBaseObj.AddBitMap(const FileName: String; const Transparent:Boolean = True);
var
  BitMap:TBitMap;
begin
  BitMap:=TBitMap.Create;
  if Transparent then begin
    BitMap.TransparentColor:=clBlack;
    BitMap.TransparentMode:=tmFixed;
    BitMap.Transparent:=True;
  end;
  BitMap.LoadFromFile(FileName);

  Items.Add(BitMap);
end; // TPLBaseObj.AddBitMap

{ TPLFloorPanels }
constructor TPLFloorPanels.Create;
begin
  inherited Create;
  dir:='floor panels';
end; // TPLFloorPanels.Create

destructor TPLFloorPanels.Destroy;
begin
  inherited Destroy;
end; // TPLFloorPanels.Destroy

{ TPLNormalPanel }
constructor TPLNormalPanel.Create;
begin
  inherited Create;
  AddBitMap(DataDir + 'vdungeon/floor panels/normal left.bmp');
  AddBitMap(DataDir + 'vdungeon/floor panels/normal right.bmp');
  AddBitMap(DataDir + 'vdungeon/floor panels/normal base.bmp');
end; // TPLNormalPanel.Create

destructor TPLNormalPanel.Destroy;
begin
  inherited Destroy;
end; // TPLNormalPanel.Destroy

procedure TPLNormalPanel.Paint(Canvas:TCanvas;  const Left, Top: Integer);
begin
  Canvas.Draw(Left, Top,Item[0]); // Line Seite
  Canvas.Draw(Left+Item[0].Width,Top, Item[1]); // Rechte Seite

  Canvas.Draw(Left, Top+Item[0].Height,Item[2]); // Davor
end;

{ TPLSpikes }
constructor TPLSpikes.Create;
begin
  inherited Create;
  AddBitMap(DataDir + 'vdungeon/floor panels/spikes left.bmp');
  AddBitMap(DataDir + 'vdungeon/floor panels/spikes right.bmp');
  AddBitMap(DataDir + 'vdungeon/floor panels/normal base.bmp');
end; // TPLSpikes.Create

destructor TPLSpikes.Destroy;
begin
  inherited Destroy;
end; // TPLSpikes.Destroy

procedure TPLSpikes.Paint(Canvas: TCanvas; const Left, Top: Integer);
begin
  Canvas.Draw(Left, Top,Item[0]); // Line Seite
  Canvas.Draw(Left+Item[0].Width,Top, Item[1]); // Rechte Seite

  Canvas.Draw(Left, Top+Item[0].Height,Item[2]); // Davor
end; // TPLSpikes.Paint

{ TPLSkeleton }
constructor TPLSkeleton.Create;
begin
  inherited Create;
  AddBitMap(DataDir + 'vdungeon/floor panels/skeleton left.bmp');
  AddBitMap(DataDir + 'vdungeon/floor panels/skeleton right.bmp');
  AddBitMap(DataDir + 'vdungeon/floor panels/normal base.bmp');
end; // TPLSkeleton.Create;

destructor TPLSkeleton.Destroy;
begin
  inherited Destroy;
end; // TPLSkeleton.Destroy

procedure TPLSkeleton.Paint(Canvas: TCanvas; const Left, Top: Integer);
begin
  // Unklar: Warum -6?
  Canvas.Draw(Left, Top-6,Item[0]); // Line Seite
  Canvas.Draw(Left+Item[0].Width,Top-6, Item[1]); // Rechte Seite
  Canvas.Draw(Left, Top+Item[0].Height-6,Item[2]); // Davor
end; // TPLSkeleton.Paint

{ TPLBigpillar }
constructor TPLBigpillar.Create;
begin
  inherited Create;
  AddBitMap(DataDir + 'vdungeon/bigpillar/big pillar left lower.bmp');
  AddBitMap(DataDir + 'vdungeon/bigpillar/big pillar right lower.bmp');
  AddBitMap(DataDir + 'vdungeon/floor panels/normal base.bmp');
end; // TPLBigpillar.Create

destructor TPLBigpillar.Destroy;
begin
  inherited Destroy;
end; // TPLBigpillar.Destroy

procedure TPLBigpillar.Paint(Canvas: TCanvas; const Left, Top: Integer);
begin
  Canvas.Draw(Left, Top-47,Item[0]); // Line Seite
  Canvas.Draw(Left+Item[0].Width,Top-47, Item[1]); // Rechte Seite

  Canvas.Draw(Left, Top+Item[0].Height-47,Item[2]); // Davor
end; // TPLBigpillar.Paint

{ TPLDoor }
constructor TPLDoor.Create;
begin
  inherited Create;
  Status:=0;
  AddBitMap(DataDir + 'vdungeon/door/door frame left pole.bmp', True); // 0
  AddBitMap(DataDir + 'vdungeon/door/door frame left.bmp', True); // 1
  AddBitMap(DataDir + 'vdungeon/door/door frame right top.bmp', True); // 2
  AddBitMap(DataDir + 'vdungeon/door/door frame right.bmp', True); // 3
  AddBitMap(DataDir + 'vdungeon/floor panels/normal base.bmp', True); // 4
  AddBitMap(DataDir + 'vdungeon/door/res00250.bmp', True); // 5
  AddBitMap(DataDir + 'vdungeon/door/res00251.bmp', True); // 6
  AddBitMap(DataDir + 'vdungeon/door/res00252.bmp', True); // 7
  AddBitMap(DataDir + 'vdungeon/door/res00260.bmp', True); // 8
  AddBitMap(DataDir + 'vdungeon/door/res00261.bmp', True); // 9
  AddBitMap(DataDir + 'vdungeon/door/res00262.bmp', True); // 10
  AddBitMap(DataDir + 'vdungeon/door/door frame right top.bmp', True); // 11
end; // TPLDoor.Create

destructor TPLDoor.Destroy;
begin
  inherited Destroy;
end; // TPLDoor.Destroy

procedure TPLDoor.Paint(Canvas: TCanvas; const Left, Top: Integer);
begin
  Canvas.Draw(Left, Top-47,Item[1]); // Linke Seite Seite
  Canvas.Draw(Left, Top+Item[0].Height-47,Item[4]); // Davor

  Canvas.Draw(Left+Item[4].Width, Top-47,Item[3]); // Rechte Seite
  Canvas.Draw(Left+Item[4].Width, Top-47-Item[11].Height,Item[11]); // Linke Obere Verlängerung

  Canvas.Draw(Left+Item[4].Width, Top-47+2,Item[7]); // Zu 1 Anfang noch unklar
  case Status of
    0: Canvas.Draw(Left+Item[4].Width, Top-47+8,Item[6]); // Auf
    1: begin
      Canvas.Draw(Left+Item[4].Width, Top-47+8,Item[7]); // Zu 2
      Canvas.Draw(Left+Item[4].Width, Top-47+16,Item[6]); // Zu 1
    end; // 1

    2: begin
      Canvas.Draw(Left+Item[4].Width, Top-47+8,Item[7]); // Zu 2
      Canvas.Draw(Left+Item[4].Width, Top-47+16,Item[7]); // Zu 2
      Canvas.Draw(Left+Item[4].Width, Top-47+24,Item[6]); // Zu 1
    end; // 2

    3: begin
      Canvas.Draw(Left+Item[4].Width, Top-47+8,Item[7]); // Zu 2
      Canvas.Draw(Left+Item[4].Width, Top-47+16,Item[7]); // Zu 2
      Canvas.Draw(Left+Item[4].Width, Top-47+24,Item[7]); // Zu 2
      Canvas.Draw(Left+Item[4].Width, Top-47+32,Item[6]); // Zu 1
    end; // 3

    4: begin
      Canvas.Draw(Left+Item[4].Width, Top-47+8,Item[7]); // Zu 2
      Canvas.Draw(Left+Item[4].Width, Top-47+16,Item[7]); // Zu 2
      Canvas.Draw(Left+Item[4].Width, Top-47+24,Item[7]); // Zu 2
      Canvas.Draw(Left+Item[4].Width, Top-47+32,Item[7]); // Zu 2
      Canvas.Draw(Left+Item[4].Width, Top-47+40,Item[6]); // Zu 1
    end; // 4

    5: begin
      Canvas.Draw(Left+Item[4].Width, Top-47+8,Item[7]); // Zu 0
      Canvas.Draw(Left+Item[4].Width, Top-47+16,Item[7]); // Zu 1
      Canvas.Draw(Left+Item[4].Width, Top-47+24,Item[7]); // Zu 2
      Canvas.Draw(Left+Item[4].Width, Top-47+32,Item[7]); // Zu 3
      Canvas.Draw(Left+Item[4].Width, Top-47+40,Item[7]); // Zu 4
      Canvas.Draw(Left+Item[4].Width, Top-47+48,Item[6]); // Zu 5
    end; // 1
  end; // Case Status
end; // TPLDoor.Paint

{ TPLKid }
constructor TPLKid.Create;
begin
  inherited Create;
  AddBitMap(DataDir + 'kid/normal.bmp'); // 0
end; // TPLKid.Create

destructor TPLKid.Destroy;
begin
  inherited Destroy;
end; // TPLKid.Destroy

procedure TPLKid.Paint(Canvas: TCanvas; const Left, Top: Integer);
begin
  Canvas.Draw(Left, Top-Item[0].Height+10,Item[0]); // Zu 5
end; // TPLKid.Paint

end.

