{
  Autor      : Michael Springwald
  Erstellt am: 22.09.2011
}

unit upllevelloader;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

type
  TPLLevelItem = record
    Key, Value, Extra:string;
  end; // TPLLevelItem

  TPLLevelLoaderRaster = class;

  TPLOnLevelLoadChar = procedure (c:char; x,y:Integer) of object;
  TPLOnLevelLoadItem = procedure (LevelItem:TPLLevelItem) of Object;

  { TPLLevelLoaderBase }
  TPLLevelLoaderBase = class
  private

  protected

  public
    FileName:string;
    constructor Create;
    destructor Destroy; override;
    procedure LoadFromFile(const aFileName:String); virtual; abstract;
    procedure SaveToFile(const aFileName:String); virtual; abstract;
  published

  end; // TPLLevelLoaderBase

    // Für die "TPLSpriteEngine". Also ungleichgroße Objekte
    TPLLevelLoader = class(TPLLevelLoaderBase)
    private

    protected

    public
      constructor Create;
      destructor Destroy; override;

      procedure LoadFromFile(const aFileName:String); override;
      procedure SaveToFile(const aFileName:String); override;

    published

    end; // TPLLevelLoader

    // Für die "TPLLevelLoaderRaster". Gleichgroße Objekte
    { TPLLevelLoaderRaster }
    TPLLevelLoaderRaster = class(TPLLevelLoaderBase)
    private
      fOnLevelLoadChar: TPLOnLevelLoadChar;
      fOnLevelLoadItem: TPLOnLevelLoadItem;
    protected
      procedure DoOnLevelLoadItem(const key, Value, Extra:String);
    public
      Level:TStringList;
      constructor Create;
      destructor Destroy; override;

      procedure LoadFromFile(const aFileName:String); override;
      procedure SaveToFile(const aFileName:String); override;
    published
      property OnLevelLoadChar:TPLOnLevelLoadChar read fOnLevelLoadChar write fOnLevelLoadChar;
      property OnLevelLoadItem:TPLOnLevelLoadItem read fOnLevelLoadItem write fOnLevelLoadItem;
    end; // TPLLevelLoaderRaster

implementation

{ TPLLevelLoaderBase }
constructor TPLLevelLoaderBase.Create;
begin
  inherited Create;
end; // TPLLevelLoaderBase.Create

destructor TPLLevelLoaderBase.Destroy;
begin
  inherited Destroy;
end; // TPLLevelLoaderBase.Destroy

constructor TPLLevelLoader.Create;
begin
  inherited Create;
end; // TPLLevelLoader.Create

destructor TPLLevelLoader.Destroy;
begin
  inherited Destroy;
end; // TPLLevelLoader.Destroy

procedure TPLLevelLoader.LoadFromFile(const aFileName: String);
begin

end; // TPLLevelLoader.LoadFromFile

procedure TPLLevelLoader.SaveToFile(const aFileName: String);
begin

end; // TPLLevelLoader.SaveToFile

procedure TPLLevelLoaderRaster.DoOnLevelLoadItem(const key, Value, Extra: String);
var
  LevelItem:TPLLevelItem;
begin
  LevelItem.Key:=key;
  LevelItem.Value:=Value;
  LevelItem.Extra:=Extra;
  OnLevelLoadItem(LevelItem)
end; // TPLLevelLoaderRaster.DoOnLevelLoadItem

{ TPLLevelLoaderRaster }
constructor TPLLevelLoaderRaster.Create;
begin
  inherited Create;
  Level:=TStringList.Create;
  fOnLevelLoadChar:=nil;
  fOnLevelLoadItem:=nil;
end; // TPLLevelLoaderRaster.Create

destructor TPLLevelLoaderRaster.Destroy;
begin
  FreeAndNil(Level);
  inherited Destroy;
end; // TPLLevelLoaderRaster.Destroy

procedure SplitString(const Str:String; var S1, S2:String; c:Char);
var
  x:Integer;
begin
  x:=Pos(c,str);
  s1:=copy(str,1,x-1);
  s2:=copy(str,x+1,Length(str));
end; // SplitString

procedure TPLLevelLoaderRaster.LoadFromFile(const aFileName: String);
var
  y,x,c, StartY:Integer;
  str,s1,s2, s3,s4:string;
  ch:Char;
begin
  FileName:=aFileName;
  if FileExists(FileName) then begin
    Level.LoadFromFile(aFileName);
    StartY:=Level.IndexOf(':Player');
    if (assigned(OnLevelLoadChar)) and (StartY > -1) then begin
      SplitString(Trim(LowerCase((Level[StartY+1]))),s1,s2,':'); // Position
      SplitString(s2,s3,s4,',');
      DoOnLevelLoadItem(s1,s3,s4);
      // ------------------------
      SplitString(Trim(LowerCase(Level[StartY+2])),s1,s2,':'); // Richtung
      DoOnLevelLoadItem(s1,s2, s3);
      // ------------------------
      SplitString(Trim(LowerCase(Level[StartY+3])),s1,s2,':'); // Länge
      DoOnLevelLoadItem(s1,s2, s3);
    end; // :player
    StartY:=Level.IndexOf(':Map')+1;
    for y:=StartY to Level.Count-1 do begin
      str:=Level[y];
      c:=Length(str);
      for x:=1 to c do begin
        ch:=str[x];
        if assigned(OnLevelLoadChar) then OnLevelLoadChar(ch,x-1,y-StartY);
      end; // for x
    end; // for y
  end
  else
    writeln('TPLLevelLoaderRaster.LoadFromFile: Datei: "', aFileName,' "gibt es nicht !!!');
end; // TPLLevelLoaderRaster.LoadFromFile

procedure TPLLevelLoaderRaster.SaveToFile(const aFileName: String);
begin

end; // TPLLevelLoaderRaster.SaveToFile

end.

