{
  Autor      : Michael Springwald
  Erstellt am: 18.09.2011
}

unit uplspriteengine;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Graphics, Controls, ExtCtrls,
  contnrs, LCLType, LCLIntf, LCLProc,
  uplimagelist, upllevelloader;

type
  TPLSpriteEngine = class;
  TPLSprite = class;
  TPLBaseSpriteEngine = class;

  TPLGameFeldBorder = (GFBNone, GFBLeft, GFBRight, GFBTop, GFBBottom);

  TPLPaintSpriteEvent = procedure (Sprite:TPLSprite; Visible:Boolean) of Object;

  { TPLSprite }
  TPLSprite = class
  private
    fBitMap: TBitmap;
    fImageItem: TPLImageListItem;
    fLeft: Integer;
    fModusImageList: Boolean;
    fOnPaintSpriteEvent: TPLPaintSpriteEvent;
    fTop: Integer;
  protected
    procedure SetLeft(const AValue: Integer); virtual;
    procedure SetTop(const AValue: Integer); virtual;
  public
    isBackupBuffer, NoBackup:Boolean;
    SpriteEngine:TPLBaseSpriteEngine;
    h, tag:Integer;
    Name:string;
    TempSprite:TPLSprite;
    constructor Create(aImageItem:TPLImageListItem; aSpriteEngine:TPLBaseSpriteEngine);
    destructor Destroy; override;

    procedure DoOnPaintSpriteEvent(Visible:Boolean);
    procedure Paint(Visible:Boolean); virtual;
    procedure Restor;
  published
    property Left:Integer read fLeft write SetLeft;
    property Top:Integer read fTop write SetTop;
    property ImageItem:TPLImageListItem read fImageItem write fImageItem;

    property OnPaintSpriteEvent:TPLPaintSpriteEvent read fOnPaintSpriteEvent write fOnPaintSpriteEvent;
    property ModusImageList:Boolean read fModusImageList write fModusImageList;
    property BitMap:TBitmap read fBitMap write fBitMap;
  end; // TPLSprite

  { TPLBaseSpriteEngine }
  TPLBaseSpriteEngine = class
  private
    fHeight: Integer;
    fLeft: Integer;
    fTop: Integer;
    fWidth: Integer;
    fImageList: TPLImageList;
    procedure SetHeight(const AValue: Integer);
    procedure SetWidth(const AValue: Integer);
  protected

  public
    Background:TBitMap;
    BackupBuffer:TBitMap;
    Buffer:TBitMap;

    Canvas:TCanvas;

    AppDir, DataDir:string;
    Timer:TTimer;
    GameViewTop:Integer;
    GameName, GameNameText:String;
    constructor Create(const aWidth, aHeight:Integer); virtual;
    destructor Destroy; override;

    procedure SetGameSize; virtual;
    // Prüft ob ein Rand erreicht wurde, wenn ja, wird zurück gegeben, welcher.
    function CheckGameFeldBorder(const x,y:Integer):TPLGameFeldBorder; virtual;

    procedure KeyDown(var Key: Word; Shift: TShiftState); virtual;
    procedure KeyUp(var Key: Word; Shift: TShiftState); virtual;
    procedure KeyPress(var Key: char); virtual;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); virtual;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); virtual;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); virtual;

    // Lädt die Grafiken
    procedure LoadImageList; virtual; abstract;

    class function Create2:TPLBaseSpriteEngine; virtual; abstract;

    procedure Paint; virtual;
  published
    property Left:Integer read fLeft write fLeft;
    property Top:Integer read fTop write fTop;
    property Width:Integer read fWidth write SetWidth;
    property Height:Integer read fHeight write SetHeight;
    property ImageList:TPLImageList read fImageList write fImageList;
  end; // TPLBaseSpriteEngine

  TSpriteEngineClass = class of TPLBaseSpriteEngine;

  { TPLSpriteEngine }
  TPLSpriteEngine = class(TPLBaseSpriteEngine)
  private
    function GetSprite(index: Integer): TPLSprite;
  protected
  public
    Sprites:TObjectList;
    constructor Create(const aWidth, aHeight: Integer); override;
    destructor Destroy; override;

    function AddSprite(const aLeft, aTop:Integer; ImageItem:TPLImageListItem):TPLSprite;
    procedure Paint; override;

    procedure PaintSpriteEvent(Sprite:TPLSprite; aVisible:Boolean);

    procedure SetGameSize; override;

    // Prüft ob eine Position, in einem anderen Sprite liegt(kein Pixel Check)
    function CheckCollision1(const x,y:Integer):TPLSprite;
    // Pixel Kollision
    function CheckCollision2(const x,y:Integer; const CheckSprite:TPLSprite):TPLSprite;

    property Sprite[index:Integer]:TPLSprite read GetSprite; default;
  published
  end; // TPLSpriteEngine

  { TPLSpriteEngineRaster }
  TPLSpriteEngineRaster = class(TPLBaseSpriteEngine)
  private

  protected
    procedure SetGameSize; override;
  public
    Map:Array of Array of TPLSprite;
    // Wie Groß ist ein Raster-Feld(Breite und Höhe)
    TileWidth, TileHeight:Integer;
    // Wie viele Tile gibt es Pro Zeile und Spalte(Horizontal und Vertikal)?
    // (Werden Automatisch Ermittelt)
    TileHoriCount, TileVertiCount:Integer;

    LevelLoaderRaster:TPLLevelLoaderRaster;
    ViewRaster:Boolean;

    constructor Create(const aWidth, aHeight, aTileWidth, aTileHeight:Integer); virtual;
    destructor Destroy; override;

    procedure InitMap;
    procedure Paint; override;
    function AddSprite(const TileX, TileY:Integer; Sprite:TPLSprite):TPLSprite;

    procedure OnLevelLoadChar(c:char; x,y:Integer); virtual;
    procedure LoadLevelFromFile(const aFileName:String); virtual;
  published

  end; // TPLSpriteEngineRaster

  { TPLGameListItem }
  TPLGameListItem = class
  private
    fData: TSpriteEngineClass;
    fGameName: String;
    fTileH: Integer;
    fTileW: Integer;
  protected

  public
    constructor Create;
    destructor Destroy; override;
  published
    property GameName:String read fGameName write fGameName;
    property Data:TSpriteEngineClass read fData write fData;
    property TileW:Integer read fTileW write fTileW;
    property TileH:Integer read fTileH write fTileH;
  end; // TPLGameListItem

  { TPLGameList }
  TPLGameList = class
  private
    function GetItem(index: Integer): TPLGameListItem;
 protected

  public
    Items:TObjectList;
    GameW, GameH:Integer;
    constructor Create;
    destructor Destroy; override;

    procedure GameAddList(const GameText:String; BaseSprite:TSpriteEngineClass; const TileW, TileH:Integer);
    property Item[index:Integer]:TPLGameListItem read GetItem;
  published
  end; // TPLGameList

  function kollision(r1,r2:TRect;B1,b2:TBitMap):boolean;

var
  GameList:TPLGameList = nil;

implementation

{
  Autor: umpani, Michael Springwald
  Update:
  Freitag den 25.Janur.2008, Freitag, 01.Febura.2008

  Diese Funktion stammt von "umpani" und ist hier zu finden:
  http://www.delphi-forum.de/viewtopic.php?t=12746&postdays=0&postorder=asc&start=20

  Ich habe sie "nur" Angepasst für mein zweck. Fragen also bitte an
  "umpani" richten im delphi-forum.de.
}

function kollision(r1,r2:TRect;B1,b2:TBitMap):boolean;
var
  ueberlapp_breite, ueberlapp_hoehe:integer;
  ueberlapp_nr1_x, ueberlapp_nr1_y, ueberlapp_nr2_x, ueberlapp_nr2_y:integer;
  x,y:integer;
  farbenr1,farbenr2:TColor;
  z:Boolean;
begin
  z:=False;
  if r1.left < r2.left then
    ueberlapp_breite := (r1.left + r1.Right) - (r2.left)
  else
    ueberlapp_breite := (r2.left + r2.Right) - r1.Left;

  if r1.Right > r2.Right then if ueberlapp_breite >= r2.Right then ueberlapp_breite := r2.Right;

  if r1.top < r2.top then
     ueberlapp_hoehe := (r1.Top + r1.Bottom) - (r2.Top)
   else
     ueberlapp_hoehe := (r2.Top + r2.Bottom) - (r1.Top);

  if r1.Bottom > r2.Bottom then if ueberlapp_hoehe > r2.Bottom then ueberlapp_hoehe := r2.Bottom;

  if (ueberlapp_breite > 0) and (ueberlapp_hoehe > 0) then begin
    if r1.Right >= r2.Right then begin
      if (r2.Left+r2.Right) >= (r1.Left+r1.Right) then begin
        ueberlapp_nr1_x := r1.Right - ueberlapp_breite;
        ueberlapp_nr2_x := 0;
      end;

      if ((r2.Left+r2.Right) < (r1.Left+r1.Right)) and (r2.left >= r1.left) then begin
        ueberlapp_nr1_x := (r2.Left-r1.Left);
        ueberlapp_nr2_x := 0;
      end;

      if (r2.Left) < (r1.Left) then begin
        ueberlapp_nr1_x := 0;
        ueberlapp_nr2_x := r2.Right - ueberlapp_breite;
      end;
    end;
  end;

  if r1.Right < r2.Right then begin
    if (r1.Left+r1.Right) >= (r2.Left+r2.Right) then begin
      ueberlapp_nr2_x := r2.Right - ueberlapp_breite;
      ueberlapp_nr1_x := 0;
    end;

    if ((r1.Left+r1.Right) < (r2.Left+r2.Right)) and (r1.Left >= r2.Left) then begin
      ueberlapp_nr2_x := (r1.Left-r2.Left);
      ueberlapp_nr1_x := 0;
    end;

    if (r1.Left < r2.Left) then begin
      ueberlapp_nr2_x := 0;
      ueberlapp_nr1_x := r1.Right - ueberlapp_breite;
    end;
  end;

  if r1.Bottom >= r2.Bottom then begin
    if (r2.Top+r2.Bottom) >= (r1.Top+r1.Bottom) then begin
      ueberlapp_nr1_y := r1.Bottom - ueberlapp_hoehe;
      ueberlapp_nr2_y := 0;
    end;

    if ((r2.Top+r2.Bottom) < (r1.top+r1.Bottom)) and (r2.Top >= r1.Top) then begin
      ueberlapp_nr1_y := (r2.top-r1.top);
      ueberlapp_nr2_y := 0;
    end;

    if (r2.top) < (r1.top) then begin
      ueberlapp_nr1_y := 0;
      ueberlapp_nr2_y := r2.Bottom - ueberlapp_hoehe;
    end;
  end;

  if r1.Bottom < r2.Bottom then begin
    if (r1.Top+r1.Bottom) >= (r2.Top+r2.Bottom) then begin
      ueberlapp_nr2_y := r2.Bottom - ueberlapp_hoehe;
      ueberlapp_nr1_y := 0;
    end;

    if ((r1.Top+r1.Bottom) < (r2.Top+r2.Bottom)) and (r1.Top >= r2.Top) then begin
      ueberlapp_nr2_y := (r1.top-r2.top);
      ueberlapp_nr1_y := 0;
    end;

    if (r1.top < r2.top) then begin
      ueberlapp_nr2_y := 0;
      ueberlapp_nr1_y := r1.Bottom - ueberlapp_hoehe;
    end;
  end;

  for x := 0 to (ueberlapp_breite-1) div 2 do begin
    for y := 0 to (ueberlapp_hoehe -1) div 2 do begin
      farbenr1 :=B1.Canvas.Pixels[ueberlapp_nr1_x+x*2,ueberlapp_nr1_y+y*2];
      farbenr2 :=B2.Canvas.Pixels[ueberlapp_nr2_x+x*2,ueberlapp_nr2_y+y*2];
      if ( farbenr1 <> b1.TransparentColor) and (farbenr2 <> b2.TransparentColor) then begin
        result:=True;
        exit;
      end;
    end; // for y
  end; // for x
  result:=False;
end; // kollision

function TPLSpriteEngine.GetSprite(index: Integer): TPLSprite;
begin
  result:=Sprites[index] as TPLSprite;
end; // TPLSpriteEngine.GetSprite

procedure TPLSpriteEngine.PaintSpriteEvent(Sprite: TPLSprite; aVisible:Boolean);
begin
  Sprite.Paint(aVisible);
end; // TPLSpriteEngine.PaintSpriteEvent

{ TPLSpriteEngine }
constructor TPLSpriteEngine.Create(const aWidth, aHeight: Integer);
begin
  inherited Create(aWidth, aHeight);
  Sprites:=TObjectList.create(True);
  Sprites.OwnsObjects:=True;
end; // TPLSpriteEngine.Create

destructor TPLSpriteEngine.Destroy;
begin
  FreeAndNil(Sprites);
  inherited Destroy;
end; // TPLSpriteEngine.Destroy

function TPLSpriteEngine.AddSprite(const aLeft, aTop: Integer; ImageItem: TPLImageListItem): TPLSprite;
begin
  result:=Sprites[Sprites.Add(TPLSprite.Create(ImageItem, Self))] as TPLSprite;
  result.OnPaintSpriteEvent:=@PaintSpriteEvent;
  Result.Left:=aLeft; Result.Top:=aTop;
end; // TPLSpriteEngine.AddSprite

procedure TPLSpriteEngine.Paint;
var
  i:Integer;
begin
  Canvas.Draw(0,0,Background);
  for i:=0 to Sprites.Count-1 do begin
    Sprite[i].isBackupBuffer:=False;
    Sprite[i].Paint(True);
  end;
end; // TPLSpriteEngine.Paint

procedure TPLSpriteEngine.SetGameSize;
begin
  inherited SetGameSize;

end; // TPLSpriteEngine.SetBackgroundSize

function TPLSpriteEngine.CheckCollision1(const x, y: Integer): TPLSprite;
var
  i:integer;
  r:TRect;
  pt:TPoint;
  s1:TPLSprite;
begin
  result:=nil; pt:=Point(x,y);
  s1:=nil;
  for i:=0 to Sprites.Count-1 do begin
    s1:=Sprite[i];
    r.Left:=s1.Left;
    r.Top:=s1.Top;
    r.Right:=r.Left+s1.ImageItem.Picture.Width;
    r.Bottom:=r.Top+s1.ImageItem.Picture.Height;
    if PtInRect(r,pt) then begin
      result:=s1;
      break;
    end;
  end; // for i
end; // TPLSpriteEngine.CheckCollision1

function TPLSpriteEngine.CheckCollision2(const x,y:Integer; const CheckSprite: TPLSprite): TPLSprite;
var
  i:integer;
  r1,r2:TRect;
  pt:TPoint;
  s1:TPLSprite;
begin
  s1:=nil; result:=nil;
  r1.Left:=x;
  r1.Top:=y;
  r1.Right:=r1.left+CheckSprite.ImageItem.Picture.Width;
  r1.Bottom:=r1.top+CheckSprite.ImageItem.Picture.Height;
  for i:=0 to Sprites.Count-1 do begin
    s1:=CheckCollision1(CheckSprite.Left,CheckSprite.Top);
    if (Assigned(s1)) and (CheckSprite <> s1) then begin
      r2.Left:=s1.Left;
      r2.Top:=s1.Top;
      r2.Right:=r2.Left+(s1.ImageItem.Picture.Width);
      r2.Bottom:=r2.Top+(s1.ImageItem.Picture.Height);
      if kollision(r2,r1,s1.ImageItem.Picture.Bitmap,CheckSprite.ImageItem.Picture.Bitmap) then begin
        result:=s1;
        break;
      end
      else
        result:=s1;
    end
    else begin
//      if (Assigned(s1)) then
  //      result:=s1;
    end;
  end; // for

end; // TPLSpriteEngine.CheckCollision2

procedure TPLSprite.SetLeft(const AValue: Integer);
begin
//  DoOnPaintSpriteEvent(False);
//  if fLeft=AValue then exit;
  fLeft:=AValue;
//  DoOnPaintSpriteEvent(True);
end; // TPLSprite.SetLeft

procedure TPLSprite.SetTop(const AValue: Integer);
begin
//  DoOnPaintSpriteEvent(False);
//  if fTop=AValue then exit;
  fTop:=AValue;
//  DoOnPaintSpriteEvent(True);
end; // TPLSprite.SetTop

constructor TPLSprite.Create(aImageItem:TPLImageListItem; aSpriteEngine:TPLBaseSpriteEngine);
begin
  inherited Create;
  tag:=0;
  TempSprite:=nil;
  h:=0;
  fLeft:=-1; fTop:=-1;
  SpriteEngine:=aSpriteEngine;
  NoBackup:=False;
  if not Assigned(aImageItem) then begin
    fModusImageList:=False;
    BitMap:=TBitmap.Create;
  end
  else begin
    ImageItem:=aImageItem;
    fModusImageList:=True;
    BitMap:=nil;
  end;

  isBackupBuffer:=False;
end; // TPLSprite.Create

destructor TPLSprite.Destroy;
begin
  inherited Destroy;
end; // TPLSprite.Destroy

procedure TPLSprite.DoOnPaintSpriteEvent(Visible:Boolean);
begin
  if Assigned(OnPaintSpriteEvent) then OnPaintSpriteEvent(self, Visible)
end; // TPLSprite.DoOnPaintSpriteEvent

procedure TPLSprite.Paint(Visible: Boolean);
begin
  if Visible then begin
    if not NoBackup then begin
      if ModusImageList then
        BitBlt(SpriteEngine.BackupBuffer.Canvas.Handle,Left, top, ImageItem.Picture.Width, ImageItem.Picture.Height,
               SpriteEngine.Canvas.Handle, SpriteEngine.Left+Left, SpriteEngine.Top+top, SRCCOPY)
      else
        BitBlt(SpriteEngine.BackupBuffer.Canvas.Handle,Left, top, BitMap.Width, BitMap.Height,
               SpriteEngine.Canvas.Handle, SpriteEngine.Left+Left, SpriteEngine.Top+top, SRCCOPY);
    end;
    isBackupBuffer:=True;

    if ModusImageList then
      SpriteEngine.Canvas.Draw(SpriteEngine.Left+Left, SpriteEngine.Top+top, ImageItem.Picture.Bitmap)
    else
      SpriteEngine.Canvas.Draw(SpriteEngine.Left+Left, SpriteEngine.Top+top, Bitmap);
  end;
end; // TPLSprite.Paint

procedure TPLSprite.Restor;
begin
  if isBackupBuffer then begin
    if ModusImageList then begin
      BitBlt(SpriteEngine.Canvas.Handle,SpriteEngine.Left+Left, SpriteEngine.Top+top,ImageItem.Picture.Width, ImageItem.Picture.Height,
             SpriteEngine.BackupBuffer.Canvas.Handle, Left, top, SRCCOPY);

      BitBlt(SpriteEngine.BackupBuffer.Canvas.Handle,Left, top,ImageItem.Picture.Width, ImageItem.Picture.Height,
             SpriteEngine.Background.Canvas.Handle, Left, top, SRCCOPY);

//      SpriteEngine.BackupBuffer.Canvas.Brush.Color:=clBlue;
    //  SpriteEngine.BackupBuffer.Canvas.FillRect(Left, Top,Left+ImageItem.Picture.Width, Top+ImageItem.Picture.Height);
    end
    else
      BitBlt(SpriteEngine.Canvas.Handle,SpriteEngine.Left+Left, SpriteEngine.Top+top,BitMap.Width, BitMap.Height,
             SpriteEngine.BackupBuffer.Canvas.Handle, Left, top, SRCCOPY);
    isBackupBuffer:=False;
  end;
end; // TPLSprite.Restor

procedure TPLBaseSpriteEngine.SetHeight(const AValue: Integer);
begin
  if fHeight=AValue then exit;
  fHeight:=AValue;
end; // TPLBaseSpriteEngine.SetHeight

procedure TPLBaseSpriteEngine.SetWidth(const AValue: Integer);
begin
  if fWidth=AValue then exit;
  fWidth:=AValue;
end; // TPLBaseSpriteEngine.SetWidth

{ TPLBaseSpriteEngine }
constructor TPLBaseSpriteEngine.Create(const aWidth, aHeight: Integer);
begin
  inherited Create;
  GameName:=''; GameNameText:='';
  Width:=aWidth;
  Height:=aHeight;
  ImageList:=TPLImageList.Create;
  Background:=TBitMap.Create;
  BackupBuffer:=TBitmap.Create;
  Buffer:=TBitMap.Create;
  Timer:=TTimer.Create(nil);
  Timer.Enabled:=False;
  Timer.Interval:=1;

  fLeft:=0; fTop:=0;
end; // TPLBaseSpriteEngine.Create

destructor TPLBaseSpriteEngine.Destroy;
begin
  FreeAndNil(Background);
  FreeAndNil(BackupBuffer);
  FreeAndNil(Buffer);
  FreeAndNil(Timer);
  ImageList.Free;
  inherited Destroy;
end; // TPLBaseSpriteEngine.Destroy

procedure TPLBaseSpriteEngine.SetGameSize;
begin
  Background.Width:=Width;
  Background.Height:=Height;

  Background.Canvas.Brush.Color:=clBlue;
  Background.Canvas.GradientFill(Rect(0,0,Width,Height),clRed,clBlue,gdHorizontal);
  //FillRect(0,0,Background.Width, Background.Height);

  BackupBuffer.Width:=Width;
  BackupBuffer.Height:=Height;
  BackupBuffer.Canvas.Brush.Color:=clBlack;
  BackupBuffer.Canvas.FillRect(0,0,Width, Height);
end; // TPLBaseSpriteEngine.SetGameSize

function TPLBaseSpriteEngine.CheckGameFeldBorder(const x, y: Integer): TPLGameFeldBorder;
begin
  result:=GFBNone;
  if X >= Width then result:=GFBRight;
  if X < 0 then result:=GFBLeft;

  if Y >= Height then result:=GFBBottom;
  if Y < 0 then result:=GFBTop;
end; // TPLBaseSpriteEngine.CheckGameFeldBorder

procedure TPLBaseSpriteEngine.KeyDown(var Key: Word; Shift: TShiftState);
begin

end; // TPLBaseSpriteEngine.KeyDown

procedure TPLBaseSpriteEngine.KeyUp(var Key: Word; Shift: TShiftState);
begin

end; // TPLBaseSpriteEngine.KeyUp

procedure TPLBaseSpriteEngine.KeyPress(var Key: char);
begin

end; // TPLBaseSpriteEngine.KeyPress

procedure TPLBaseSpriteEngine.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

end; // TPLBaseSpriteEngine.MouseDown

procedure TPLBaseSpriteEngine.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

end; // TPLBaseSpriteEngine.MouseUp

procedure TPLBaseSpriteEngine.MouseMove(Shift: TShiftState; X, Y: Integer);
begin

end; // TPLBaseSpriteEngine.MouseMove

procedure TPLBaseSpriteEngine.Paint;
begin

end; // TPLBaseSpriteEngine.Paint

procedure TPLSpriteEngineRaster.SetGameSize;
begin
  TileHoriCount:=(Width div TileWidth);
  TileVertiCount:=(Height div TileHeight)+1;
//  writeln(TileHoriCount,'X',TileVertiCount);
  InitMap;
  inherited SetGameSize;
end; // TPLSpriteEngineRaster.SetGameSize

{ TPLSpriteEngineRaster }
constructor TPLSpriteEngineRaster.Create(const aWidth, aHeight, aTileWidth, aTileHeight:Integer);
begin
  inherited Create(aWidth, aHeight);
  TileWidth:=aTileWidth; TileHeight:=aTileHeight;
  Width:=aWidth; Height:=aHeight;
  LevelLoaderRaster:=TPLLevelLoaderRaster.Create;
  LevelLoaderRaster.OnLevelLoadChar:=@OnLevelLoadChar;
  ViewRaster:=False;
end; // TPLSpriteEngineRaster.Create

destructor TPLSpriteEngineRaster.Destroy;
var
  i:Integer;
begin
  InitMap; FreeAndNil(LevelLoaderRaster);
  for i:=0 to TileVertiCount do
    SetLength(Map[i],0);
  SetLength(map,0);
  inherited Destroy;
end; // TPLSpriteEngineRaster.Destroy

procedure TPLSpriteEngineRaster.InitMap;
var
  x,y:Integer;
begin
  SetLength(map, TileVertiCount+1, TileHoriCount+1);

  for y:=0 to TileVertiCount do begin
    for x:=0 to TileHoriCount do begin
      if Assigned(Map[y,x]) then Map[y,x].Free;
      Map[y,x]:=nil;
    end;
  end;
end; // TPLSpriteEngineRaster.InitMap

procedure TPLSpriteEngineRaster.Paint;
var
  x,y,l,px,py, p1:Integer;
begin
  Canvas.Draw(Left,Top, Background);
  BackupBuffer.Canvas.Draw(0,0, Background);
  px:=0; py:=0;
  for y:=0 to TileVertiCount do begin
    for x:=0 to TileHoriCount do begin
      if Assigned(Map[y,x]) then begin
        Map[y,x].isBackupBuffer:=False;
        Map[y,x].paint(True);
      end;
      if ViewRaster then begin
        Canvas.Brush.Style:=bsClear;
        Canvas.Pen.Color:=clYellow;
        Canvas.Rectangle(x * TileWidth,y * TileHeight,(x * TileWidth)+TileWidth,(y * TileHeight)+TileHeight);

        Background.Canvas.Brush.Style:=bsClear;
        Background.Canvas.Pen.Color:=clYellow;
        Background.Canvas.Rectangle(px,py,px,py+TileHeight);
        px:=px+TileWidth;
      end;
    end; // for x
    px:=0;
    py:=py+TileHeight;
  end; // for y
end; // TPLSpriteEngineRaster.Paint

function TPLSpriteEngineRaster.AddSprite(const TileX, TileY: Integer; Sprite:TPLSprite): TPLSprite;
begin
  map[TileY, TileX]:=Sprite;
  map[TileY, TileX].Left:=TileX*TileWidth;
  map[TileY, TileX].Top:=(TileY*TileHeight);
  result:=map[TileY, TileX];
end; // TPLSpriteEngineRaster.AddSprite

procedure TPLSpriteEngineRaster.OnLevelLoadChar(c: char; x, y: Integer);
begin

end; // TPLSpriteEngineRaster.OnLevelLoadChar

procedure TPLSpriteEngineRaster.LoadLevelFromFile(const aFileName: String);
begin
  LevelLoaderRaster.LoadFromFile(aFileName);
end; // TPLSpriteEngineRaster.LoadLevelFromFile

function TPLGameList.GetItem(index: Integer): TPLGameListItem;
begin
  result:=TPLGameListItem(Items[index]);
end; // TPLGameList.GetItem

{ TPLGameList }
constructor TPLGameList.Create;
begin
  inherited Create;
  Items:=TObjectList.Create;
end; // TPLGameList.Create;

destructor TPLGameList.Destroy;
begin
  FreeAndNil(Items);
  inherited Destroy;
end; // TPLGameList.Destroy

procedure TPLGameList.GameAddList(const GameText: String;BaseSprite: TSpriteEngineClass;const TileW, TileH:Integer);
var
  GI:TPLGameListItem;
begin
  GI:=TPLGameListItem.Create;
  GI.GameName:=GameText;
  GI.Data:=BaseSprite;
  GI.TileW:=TileW;
  GI.TileH:=TileH;
  items.Add(GI);
end; // TPLGameList.GameAddList

{ TPLGameListItem }
constructor TPLGameListItem.Create;
begin
  inherited Create;
  fGameName:='';
end; // TPLGameListItem.Create

destructor TPLGameListItem.Destroy;
begin
  inherited Destroy;
end; // TPLGameListItem.Destroy

initialization
  GameList:=TPLGameList.Create;

finalization
  FreeAndNil(GameList);

end.

